"""Template tags for viewpass."""

from django import template

from viewpass.util import get_viewpass_url

register = template.Library()


@register.simple_tag(takes_context=True)
def viewpass_url(context, perm, path=None):
    """Return a URL path for current path, viewpass-encoded with permission.

    Args:
        context (Context): context of template tag
        perm (str): permission to grant
        path (str): Path to encode; if none given, uses current path

    Returns:
        str: URL path with embedded viewpass information.

    To use this, in a template:

    .. code-block:: jinja

       {% viewpass_url 'View' %}

    This would return a URL with embedded, cryptographically-signed information about the
    `View` permission in it
    """

    if path is None:
        path = context.request.get_full_path()

        # Don't re-viewpass an existing viewpass URL
        if "viewpass=" in path:
            return path

    return get_viewpass_url(path, perm)



"""Authentication backend to support viewpass."""

from django.contrib.auth.backends import ModelBackend


class ViewPassBackend(ModelBackend):
    """Authentication backend that checks for viewpass permission.

    This is the heart of viewpass, used when a viewpass-visiting-user has a
    permission check.

    This checks to see if the permission being sought is the one
    stored on the user as `_viewpass_perm` (this storing is done earlier by
    the middleware).

    To be used, this must be added to your config's `AUTHENTICATION_BACKENDS`.
    """

    def has_perm(self, user_obj, perm, obj=None):
        """Does this user have permission for this object? (check viewpass).

        Args:
            user_obj (User): Django user object.
            perm (str): String name of permission.
            obj (Optional): (Ignored by us).

        Returns:
            bool: does user have permission?
        """

        return perm == getattr(user_obj, '_viewpass_perm', None)



"""Utility code for viewpass."""

from django.core import signing


def get_viewpass_url(path, perm):
    """Returns viewpass encoded path for this path and permission.

    Args:
        path (str): site path you want a token for. For example, '/report/'
        perm (str): permission you want a token for. For example, 'reporting.view_report'

    Returns:
        str: path with a "viewpass" GET request token.
            This token has a crypto-signed version of the path/permission so, when this
            URL is used, the viewpass middleware can trust this and give the user
            permission to visit that URL.
    """

    token = signing.dumps([path, perm])

    if "?" in path:
        return path + "&viewpass=" + token

    else:
        return path + "?viewpass=" + token


#!/usr/bin/env python

import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
        name='django-viewpass',
        version='1.0.5',
        packages=['viewpass', 'viewpass.templatetags'],
        include_package_data=True,
        license='GPL',
        description='A Django app to provide access to protected resources via URL tokens.',
        long_description=README,
        url='https://github.com/joelburton/viewpass.git',
        download_url='https://github.com/joelburton/django-viewpass/tarball/1.0.5',
        author='Joel Burton',
        author_email='joel@joelburton.com',
        classifiers=[
            'Environment :: Web Environment',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Topic :: Internet :: WWW/HTTP',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
)



"""Middleware to support viewpass temporary-access tokens."""

from django.core import signing
from django.core.exceptions import SuspiciousOperation


class ViewPassMiddleware(object):
    """Handles when viewpass is given in the request.

    If the user is presenting a viewpass token in the URL, check its
    authenticity and, if good, store the permission-to-be-granted on the user
    object.
    """

    # noinspection PyMethodMayBeStatic
    def process_request(self, request):
        """Grab viewpass information from request, if present.

        Args:
            request (Request): request in process; this is marked-up with permission granted

        Returns:
            None
        """

        # Look on URL for "viewpass" parameter -- if it's there , the user is
        # trying to use a viewpass token

        viewpass = request.GET.get('viewpass')

        if viewpass:

            # The viewpass token is a signed object; use the crypto library to
            # check the signature and decompose into the original object.
            # This will raise an exception if the signature is forged or the
            # object is corrupted -- which is what we'd want to happen.

            info = signing.loads(viewpass)
            path, permission = info

            our_path = request.get_full_path()

            # Strip off ?viewpass=... or &viewpass=... from the end
            # (we know it will be at the end, since we put it there in our tag!)
            our_path = our_path[:our_path.find("viewpass=")-1]

            if path != our_path:
                raise SuspiciousOperation(
                        "Viewpass path %s does not match real %s" % (path, our_path))

            # noinspection PyBroadException
            try:
                user = request.user

                # Add onto the user the permission they'll have for this request.
                user._viewpass_perm = permission

            except Exception:
                # Ugh, a bare exception -- but there are so many things that
                # could happen in weird configurations in trying to get a user
                # object at this point, so better safe than sorry.
                pass

        # We do not return a request; doing so would say that we want to skip
        # all other middleware and Django processing. Instead, we return None.


